Node tree;
int max_fanout = 5; // maximum node fanout; must be >= 0
int max_level = 5; // maximum tree depth; must be >= 1
int diameter = 2400; // diameter of the universe 
int jump = -8; // expansion/contraction jump


void setup() {
  fullScreen();
  tree = new Node(width/2, height/2, 1, max_fanout, 0, 0);
}

void draw() {
  background(255);
  tree.picture();
  tree.update();
}

void mouseReleased() {
  tree = new Node(width/2, height/2, 1, max_fanout, 0, 0);
} 


class Node {
  float x, y; // node screen coordinates
  float angle, radius; // node polar coordinates wrt parent node
  int level; // node level
  int fanout; // node fanout
  float rotation; // node rotation speed and direction
  Node[] children = {}; // list of children

  Node(float _x, float _y, int _level, int _fanout, float _angle, float _radius) {
    x = _x;
    y = _y;
    level = _level;
    fanout = _fanout;
    angle = _angle;
    radius = _radius;
    rotation = radians(random(-1, 1));     

    if (level < max_level) { // create children
      children = new Node[fanout];
      for (int i = 0; i < children.length; i++) {
        int clevel = level + 1;
        int cfanout;
        if (level < (max_level - 1)) { // node with internal children
          cfanout = max_fanout;
        } else { // node with leaf children
          cfanout = 0;
        }
        // same radius for all nodes
        float cradius = diameter/3; 
        float cangle = random(0, TWO_PI);
        float cx = x + cos(cangle) * cradius;
        float cy = y + sin(cangle) * cradius;
        children[i] = new Node(cx, cy, clevel, cfanout, cangle, cradius);
      }
    }
  }


  void picture() {
    fill(0);
    ellipse(x, y, 20 / level, 20 / level); // draw node
    if (level < max_level) { // draw edges
      for (int i = 0; i < children.length; i++) { 
        stroke(0);
        strokeWeight(5 / level);
        line(x, y, children[i].x, children[i].y); 
        children[i].picture();
      }
    }
  }

  void update() {
    for (int i = 0; i < children.length; i++) {
      children[i].angle += children[i].rotation;
      // change radius
      children[i].radius += jump;
      if (children[i].radius < 0 || children[i].radius > diameter) {
        jump = -jump;
      }
      children[i].x = x + cos(children[i].angle) * children[i].radius;
      children[i].y = y + sin(children[i].angle) * children[i].radius;
      children[i].update();
    }
  }
}